// For more info on plugins, parameters and how to use them refer to the
// V-Ray AppSDK documentation.

#define VRAY_RUNTIME_LOAD_PRIMARY

#include "vraysdk.hpp"
#include "vrayplugins.hpp"

#include "utils.h"

using namespace VRay;
using namespace VRay::Plugins;
using namespace std;

const char *BASE_PATH = getenv("VRAY_SDK");
string SCENE_PATH = (BASE_PATH ? string(BASE_PATH) : string(".")) + PATH_DELIMITER + "scenes";

void onStateChanged(VRayRenderer &renderer, RendererState oldState, RendererState newState, double instant, void* userData) {
	if (oldState == IDLE_INITIALIZED && newState == PREPARING) {
		printf("Sequence started.\n");
	}
	if (newState == IDLE_FRAME_DONE || newState == IDLE_DONE) {
		printf("Image Ready, frame %d (sequenceEnded = %d)\n", renderer.getCurrentFrame(), renderer.isSequenceEnded());

		if (newState == IDLE_FRAME_DONE) {
			// If the sequence has NOT finished - continue with next frame.
			renderer.continueSequence();
		} else {
			printf("Sequence done. Closing renderer...\n");
		}
	}
}

int main() {
	// Change process working directory to SCENE_PATH in order to be able to
	// load relative scene resources.
	changeCurrentDir(SCENE_PATH.c_str());

	// Load V-Ray SDK library.
	// The directory containing the vray shared object should be present in
	// the PATH environment variable.
	VRayInit init(NULL, true);

	// Create an instance of VRayRenderer with default options.
	// The renderer is automatically closed at the end of the current scope.
	VRayRenderer renderer;

	// It's recommended to always have a console log
	renderer.setOnLogMessage(logMessage);

	// Add a listener for the renderer state change event.
	// We will use it to detect when a frame is completed and allow the next one to start.
	renderer.setOnStateChanged(onStateChanged);

	// Load scene from a file.
	renderer.load("Sequence_ifl.vrscene");

	// Maximum paths per pixel for interactive mode.
	// Set a low sample level to complete rendering faster.
	// The default value is unlimited.
	// We have to call this *after* loading the scene.
	renderer.setInteractiveSampleLevel(1);

	// Start rendering a sequence of frames (animation).
	renderer.renderSequence();

	// Wait until the entire sequence has finished rendering.
	renderer.waitForSequenceEnd();

	return 0;
}
